<?php
/**
 * This file is part of Exponent Content Management System
 *
 * Exponent is free software; you can redistribute
 * it and/or modify it under the terms of the GNU
 * General Public License as published by the Free
 * Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * @category   Exponent CMS
 * @package    Framework
 * @subpackage Controllers
 * @author     Adam Kessler <adam@oicgroup.net>
 * @copyright  2004-2009 OIC Group, Inc.
 * @license    GPL: http://www.gnu.org/licenses/gpl.txt
 * @version    Release: @package_version@
 * @link       http://www.exponent-docs.org/api/package/PackageName
 */

class expCommentController extends expController {
    public $base_class = 'expComment';
    protected $add_permissions = array('approve'=>"Approve Comments");
   	protected $remove_permissions = array('edit', 'create');

	function name() { return $this->displayname(); } //for backwards compat with old modules
    function displayname() { return "Comments"; }
    function description() { return "Use this module to add comments to a page."; }
    function author() { return "Adam Kessler @ OIC Group, Inc"; }
    function hasSources() { return true; }
    function hasViews() { return true; }

	function edit() {
	    $id = empty($this->params['id']) ? null : $this->params['id'];
	    $comment = new expComment($id);
		assign_to_template(array('comment'=>$comment));
	}	
	
	function manage() {
	    expHistory::set('managable', $this->params);
	    
	    $sql  = 'SELECT c.* FROM '.DB_TABLE_PREFIX.'_expComments c ';
        $sql .= 'JOIN '.DB_TABLE_PREFIX.'_content_expComments cnt ON c.id=cnt.expcomments_id ';
        $sql .= 'WHERE cnt.content_id='.$this->params['content_id']." AND cnt.content_type='".$this->params['content_type']."' ";
        $sql .= 'AND c.approved=0';
        
        $page = new expPaginator(array(
            'model'=>'expComment',
            'sql'=>$sql, 
            'limit'=>10,
            'order'=>'created_at',
            'controller'=>$this->baseclassname,
            'action'=>$this->params['action'],
            'columns'=>array('Approved'=>'approved', 'Poster'=>'name', 'Comment'=>'body'),
        ));
        
        assign_to_template(array('page'=>$page));
	}
	
	function getComments() {
		global $user, $db;

        $sql  = 'SELECT c.* FROM '.DB_TABLE_PREFIX.'_expComments c ';
        $sql .= 'JOIN '.DB_TABLE_PREFIX.'_content_expComments cnt ON c.id=cnt.expcomments_id ';
        $sql .= 'WHERE cnt.content_id='.$this->params['content_id']." AND cnt.content_type='".$this->params['content_type']."' ";
        $sql .= 'AND c.approved=1';
        
        $comments = new expPaginator(array(
            'model'=>'expComment',
            'sql'=>$sql, 
            'limit'=>10,
            'order'=>'created_at',
            'controller'=>$this->baseclassname,
            'action'=>$this->params['action'],
            'columns'=>array('Readable Column Name'=>'Column Name'),
        ));
        
        // count the unapproved comments
        if (COMMENTS_REQUIRE_APPROVAL == 1 && $user->isAdmin()) {
            $sql  = 'SELECT count(com.id) as c FROM '.DB_TABLE_PREFIX.'_expComments com ';
            $sql .= 'JOIN '.DB_TABLE_PREFIX.'_content_expComments cnt ON com.id=cnt.expcomments_id ';
            $sql .= 'WHERE cnt.content_id='.$this->params['content_id']." AND cnt.content_type='".$this->params['content_type']."' ";
            $sql .= 'AND com.approved=0';
            $unapproved = $db->countObjectsBySql($sql);
        } else {
            $unapproved = 0;
        }        
        
        assign_to_template(array(
            'comments'=>$comments,
            'unapproved'=>$unapproved, 
			'content_id'=>$this->params['content_id'], 
			'content_type'=>$this->params['content_type'],
			'user'=>$user,
			'hideform'=>$this->params['hideform'],
			'hidecomments'=>$this->params['hidecomments'],
			'title'=>$this->params['title'],
			'formtitle'=>$this->params['formtitle'],
		));
	}

    function update() {
        global $db, $user;
        
        // check the anti-spam control
        expValidator::check_antispam($this->params, "Your comment could not be posted becaues anti-spam verification failed.  Please try again.");
        
        // figure out the name and email address
        if (!empty($user->id) && empty($this->params['id'])) {
            $this->params['name'] = $user->firstname." ".$user->lastname;
            $this->params['email'] = $user->email;
        }
        
        // if comments are configed to require approvals set this to 0 otherwise we 
        // will just go ahead and turn the approval on for this comment.
        $this->expComment->approved = (COMMENTS_REQUIRE_APPROVAL == 1 && !$user->isAdmin()) ? 0 : 1;
        
        // save the comment
        $this->expComment->update($this->params);
        
        // attach the comment to the datatype it belongs to (blog, news, etc..);
		$obj->content_type = $this->params['content_type'];
		$obj->content_id = $this->params['content_id'];
		$obj->expcomments_id = $this->expComment->id;
		if(isset($this->params['subtype'])) $obj->subtype = $this->params['subtype'];
		$db->insertObject($obj, $this->expComment->attachable_table);
		
		$msg = 'Thank you for posting a comment.';
		if (COMMENTS_REQUIRE_APPROVAL == 1 && !$user->isAdmin()) {
		    $msg .= ' Your comment is now pending approval. You will receive an email to ';
		    $msg .= $this->expComment->email.' letting you know when it has been approved.'; 
		}
		
		if (COMMENTS_REQUIRE_NOTIFICATION && !$user->isAdmin()) {
		    $this->sendNotification($this->expComment);
		}
		
		flash('message', $msg);
		
		expHistory::back();
	}
	
	public function approve() {
	    expHistory::set('editable', $this->params);
	    
	    if (empty($this->params['id'])) {
	        flash('error', 'No ID supplied for comment to approve');
	        expHistory::back();
	    }
	    
	    $comment = new expComment($this->params['id']);
	    assign_to_template(array('comment'=>$comment));
	}
	
	public function approve_submit() {
	    if (empty($this->params['id'])) {
	        flash('error', 'No ID supplied for comment to approve');
	        expHistory::back();
	    }
	    
	    $comment = new expComment($this->params['id']);
	    $comment->body = $this->params['body'];
	    $comment->approved = $this->params['approved'];
	    $comment->save();
	    expHistory::back();
	}
	
	public function approve_toggle() {
	    if (empty($this->params['id'])) return;
	    $comment = new expComment($this->params['id']);
	    $comment->approved = $comment->approved == 1 ? 0 : 1;
	    $comment->save();
	    expHistory::back();
	}
	public function delete() {
	    global $db;
	    
	    if (empty($this->params['id'])) {
	        flash('error', 'Missing id for the comment you would like to delete');
	        expHistory::back();
	    }
	    
	    // delete the comment
        $comment = new expComment($this->params['id']);
        $rows = $comment->delete();
        
        // delete the assocication too
        $db->delete($comment->attachable_table, 'expcomments_id='.$this->params['id']);        
        
        // send the user back where they came from.
        expHistory::back();
	}
	
	private function sendNotification($comment) {
	    if (empty($comment)) return false;
	    
	    // setup some email variables.
	    $subject = 'Notification of a New Comment Posted to '.URL_BASE;
        $tos = split(',', str_replace(' ', '', COMMENTS_NOTIFICATION_EMAIL));
        $editlink = makelink(array('controller'=>'expComment', 'action'=>'edit', 'id'=>$comment->id));
        
        // make the email body
        $body  = 'Posted By:&nbsp;'.$comment->name."<br>";
        $body .= 'Posters Email:&nbsp;'.$comment->email."<br><br>";
        $body .= $comment->body."<br><br>";
        $body .= 'You can view, edit and optionally approved this comment by going to ';
        $body .= '<a href="'.$editlink.'">'.$editlink.'</a>';
        
        // create the mail message
        $mail = new expMail();        
        $mail->quickSend(array(
                'html_message'=>$body,
			    'to'=>$tos,
			    'from'=>trim(SMTP_FROMADDRESS),
			    'from_name'=>trim(URL_BASE),
			    'subject'=>$subject,
        ));
        
        return true;
	}
}

?>
