<?php

##################################################
#
# Copyright (c) 2007-2008 OIC Group, Inc.
# Created by Adam Kessler @ 09/06/2007
#
# This file is part of Acorn Web API
#
# Exponent is free software; you can redistribute
# it and/or modify it under the terms of the GNU
# General Public License as published by the Free
# Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# GPL: http://www.gnu.org/licenses/gpl.txt
#
##################################################

class orderitem extends expRecord {
	public $table = 'orderitems';
	public $has_one = array('shippingmethod');
	public $opts = array();
	
	function __construct($params=array()) {
		global $db, $user;
		
		if (!empty($params['id']) || is_numeric($params)) {
			parent::__construct($params);
			$prodtype = $this->product_type;
			$this->product = new $prodtype($this->product_id);
		} elseif (isset($params['product_id']) && isset($params['product_type'])) {	
			// see if this is an existing item in the cart
			$order = order::getUserCart();
			
			$where = 'orders_id='.$order->id.' AND product_id='.$params['product_id']." AND product_type='".$params['product_type']."'";
			$where .= empty($params['options']) ? '' : " AND options='".$params['options']."'";
			
			$item = $db->selectObject($this->table, $where);
    
			$this->product = new $params['product_type']($params['product_id']);
			if (empty($item)) {
				parent::__construct(array(
        	        'orders_id'=>$order->id,
                	'user_id'=>$user->id,
                    'product_id'=>$this->product->id,
                    'product_type'=>$params['product_type'],
	                'products_name'=>$this->product->title,
        	        'products_price'=>$this->product->getPrice(),
                	'quantity'=>$this->product->getDefaultQuantity()
                ));
            } else {
	            parent::__construct($item->id);
            	$this->quantity = $this->product->incrementQuantity($this->quantity);
            }
	    } else {
		    parent::__construct();
	    }
	    if (isset($this->options)) {
	        $this->opts = expUnserialize($this->options);
	    }
	}

    public function merge($params) {        
        // check to see if this item was in the old cart we are merging..if so we will 
        // up tick the quantity...otherwise we will just add the item to the cart.
        $existing_item = $this->find('first', "orders_id=".$params['orders_id']." AND product_id=".$this->product_id." AND product_type='".$this->product_type."' AND options='".$this->options."'");
        if (empty($existing_item)) {
            $this->update(array('orders_id'=>$params['orders_id'], 'user_id'=>$params['user_id']));
        } else {
            $existing_item->update(array('quantity'=>$existing_item->quantity + $this->quantity));
        }
        
    }
    
	public function getTotal() {
	    // because these variable names are so long we're going to set these for 
	    // shorthand/readability purposes
	    $prod = $this->product;
	    $quantity_amount = $prod->quantity_discount_num_items;
	    
	    if ($this->quantity <= $quantity_amount) {
	        return $this->products_price * $this->quantity;
	    } else {
	        // if this is set we only apply the discount to the products over the limit.
	        if ($prod->quantity_discount_apply) {
                $disc_priced = $this->quantity - $prod->quantity_discount_num_items;
                $orig_priced = $this->quantity - $disc_priced;
            } else {
                $disc_priced = $this->quantity;
                $orig_priced = 0;
            }
	        
	        $total =  $this->products_price * $orig_priced ;
	        if ($prod->quantity_discount_amount_mod == "$") {
	            $total += ($this->products_price - $prod->quantity_discount_amount) * $disc_priced;
	        } elseif ($prod->quantity_discount_amount_mod == "%") {
	            $subtotal = $this->products_price * $disc_priced;
	            $total += ($subtotal - (($this->products_price * $disc_priced) * ($prod->quantity_discount_amount * .01)));
	        }
	    }
	    return $total;
	}
	
	public function getExtraData() {
	    $product = new $this->product_type($this->product_id);
	    return $product->formatExtraData($this);
	}
}
?>
