<?php

##################################################
#
# Copyright (c) 2004-2006 OIC Group, Inc.
# Created by Adam Kessler @ 05/28/2008
#
# This file is part of Exponent
#
# Exponent is free software; you can redistribute
# it and/or modify it under the terms of the GNU
# General Public License as published by the Free
# Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# GPL: http://www.gnu.org/licenses/gpl.txt
#
##################################################

class cartController extends expController {
	public $basemodel_name = 'order';
	private $checkout_steps = array('productinfo', 'specials', 'form', 'wizards', 'newsletter', 'confirmation', 'postprocess');
	public $useractions = array('show'=>'Display Cart');

	function name() { return $this->displayname(); } //for backwards compat with old modules
	function displayname() { return "Ecommerce Shopping Cart"; }
	function description() { return "This is the cart users will add products from your store to."; }
	function author() { return "Adam Kessler @ OIC Group, Inc"; }
	function hasSources() { return true; }
	function hasViews() { return true; }
	function hasContent() { return true; }
	function supportsWorkflow() { return false; }

	function addItem() {
	    global $router;
	    // call this product types addToCart callback function
	    $product_type = isset($this->params['product_type']) ? $this->params['product_type'] : 'product';
	    $product = new $product_type($this->params['product_id']);
	    if (!$product->addToCart($this->params)) {
	        // if we hit here it means this product type was missing some
	        // information it needs to add the item to the cart..so we need to help
	        // it display it's addToCart form
	        redirect_to(array(
	                'controller'=>'cart',
	                'action'=>'displayForm',
	                'form'=>'addToCart',
	                'product_id'=>$this->params['product_id'],
	                'product_type'=>$this->params['product_type'],
	        ));
	    } 
	    
	    if (empty($this->params['quick'])) {
	        flash('message', "Added ".$product->title." to your cart. <a href='" . $router->makeLink(array('controller'=>'cart', 'action'=>'checkout'), false, true) ."'>Click here to checkout now.</a>");
	        expHistory::back();
	    } else {
	        redirect_to(array('controller'=>'cart', 'action'=>'quickPay'));
	    }
	        
	}
	
	function displayForm() {
	    $product_type = isset($this->params['product_type']) ? $this->params['product_type'] : 'product';
	    $product = new $product_type($this->params['product_id']);
	    $form = new controllerTemplate($this, $product->getForm($this->params['form']));
	    $form->assign('params', $this->params);
	    $form->assign('product', $product);
	    echo $form->render();
	}
	    
	function updateQuantity() {
		global $order;
		if (exponent_javascript_inAjaxAction()) {
			$id = str_replace('quantity-', '', $this->params['id']);
            $item = new orderitem($id);
            if (!empty($item->id)) {
                $newqty = $item->product->updateQuantity($this->params['value']);
                if ($newqty > $item->product->quantity) {
                    if ($item->product->availability_type == 1) {
                        $diff = ($item->product->quantity <=0) ? $newqty : $newqty - $item->product->quantity;
                        $updates->message = 'Only '.$item->product->quantity.' '.$item->products_name.' are currently in stock. Shipping may be delayed on the other '.$diff;
                    } elseif ($item->product->availability_type == 2) {
                        $updates->message = $item->products_name.' only has '.$item->product->quantity.' on hand. You can not add any more to your cart.';
                        $updates->cart_total = '$'.number_format($order->getCartTotal(), 2);
			            $updates->item_total = '$'.number_format($item->getTotal(), 2);
			            $updates->item_id = $id;
			            $updates->quantity = $item->product->quantity;
			            echo json_encode($updates);
			            return true;
                    }
                }
			    $item->quantity = $newqty;
			    $item->save();
			    $order->refresh();
			    $updates->cart_total = '$'.number_format($order->getCartTotal(), 2);
			    $updates->item_total = '$'.number_format($item->getTotal(), 2);
			    $updates->item_id = $id;
			    $updates->quantity = $item->quantity;
			    echo json_encode($updates);
			}
		} else {
			redirect_to(array('controller'=>'cart','action'=>'show'));
		}
	}
	
	function removeItem() {
		global $order;
		foreach ($order->orderitem as $item) {
			if ($item->id == intval($this->params['id'])) {
			    $product = new  $item->product_type($item->product_id);
			    $product->removeItem($item);
				$item->delete();
			}
		}

		expHistory::back();
	}

	function show() {
		global $order;
		expHistory::set('viewable', $this->params);
		assign_to_template(array('items'=>$order->orderitem, 'order'=>$order));
	}
	
	function checkout() {
		global $user, $order;

		if (!exponent_users_isLoggedIn()) {
		    expHistory::set('viewable', $this->params);
			flash('message', "Please login to continue the checkout process.");
			exponent_flow_redirecto_login(makeLink(array('module'=>'cart','action'=>'checkout'), 'secure'));
		}

		if (empty($order->orderitem)) flashAndFlow('error', 'There are no items in your cart.');

        $billing = new billing();
        if (count($billing->available_calculators) < 1) {
            flashAndFlow('error', 'This store is not configured to allow checkouts yet.  Please try back soon.');
        }
        
        // set a flow waypoint
		expHistory::set('viewable', $this->params);
		
		// call each products checkout() callback & calculate total
		foreach($order->orderitem as $item) {
			$product = new $item->product_type($item->product_id);
			$product->checkout();
		}

		// get the specials...this is just a stub function for now.
		$specials = $this->getSpecials();	

		// get all the necessary addresses..shipping, billing, etc
		$address = new address();
		$addresses_dd = $address->dropdownByUser($user->id);
		$addresses = $address->find('all', 'user_id='.$user->id);
		if (empty($addresses)) {
		    flash('error', 'There does not appear to be any addresses in your address book yet.  
		    You need at least one address to continue checking out.  Please add your primary address now.
		    <br><br>
		    If you need to add another billing or shipping address you will be able to do so on the following page.
		    ');
		    redirect_to(array('controller'=>'address','action'=>'edit'));
		}

		// get the shipping calculators and the shipping methods if we need them
		$shipping = new shipping();
		$shipping->getRates();
		
		assign_to_template(array(
					'addresses_dd'=>$addresses_dd,
					'addresses'=>$addresses,
					'shipping'=>$shipping,
					'user'=>$user,
					'billing'=>$billing,
					//'billingcalcs'=>$billingcalcs,
					'order'=>$order,
					//'needs_address'=>$needs_address,
		));
	}

	public function confirm() {
		global $order, $user, $db;
		
		// get the shippnig and billing objects, these objects handle the setting up the billing/shipping methods
		// and their calculators
		$shipping = new shipping();
		$billing = new billing();

		// Make sure all the pertanent data is there...otherwise flash an error and redirect to the checkout form.
		if (!$user->isLoggedIn()) {
            flash('message', "It appears that your session has expired. Please log in to continue the checkout process.");
            exponent_flow_redirecto_login(makeLink(array('module'=>'cart','action'=>'checkout'), 'secure'));
        }
        
		if (empty($order->orderitem)) flash('error', 'There are no items in your cart.');
		if (empty($shipping->calculator->id) && !$shipping->splitshipping) flash('error', 'You must pick a shipping method');
		if (empty($shipping->address->id) && !$shipping->splitshipping) flash('error', 'You must pick a shipping address');
		if (empty($billing->calculator->id)) flash('error', 'You must pick a billing method');
		if (empty($billing->address->id)) flash('error', 'You must select a billing address');
        
        // make sure all the methods picked for shipping meet the requirements
        foreach ($order->getShippingMethods() as $smid) {
            $sm = new shippingmethod($smid);
            $calcname = $db->selectValue('shippingcalculator', 'calculator_name', 'id='.$sm->shippingcalculator_id);
            $calc = new $calcname($sm->shippingcalculator_id);
            $ret = $calc->meetsCriteria($sm);
            if (is_string($ret)) flash('error', $ret);
        }
        
        // if we encounterd any errors we will return to the checkout page and show the errors
   		if (!expQueue::isQueueEmpty('error')) redirect_to(array('controller'=>'cart', 'action'=>'checkout'));
        
        // get the billing options..this is usually the credit card info entered by the user
		$opts = $billing->calculator->userFormUpdate($this->params);
		exponent_sessions_set('billing_options', $opts);
		
		// final the cart totals
		$order->calculateFinalTotal();

		assign_to_template(array(
		    'shipping'=>$shipping, 
		    'billing'=>$billing, 
		    'order'=>$order,
		    'total'=>$order->total, 
		    'billinginfo'=>$billing->calculator->userView($opts),
		));
	}

	public function process() {
		global $db, $order, $user;
		if (!$user->isLoggedIn() && empty($this->params['nologin'])) {
            flash('message', "It appears that your session has expired. Please log in to continue the checkout process.");
            exponent_flow_redirecto_login(makeLink(array('module'=>'cart','action'=>'checkout'), 'secure'));
        }
		// if this error hits then something went horribly wrong or the user has tried to hit this 
		// action themselves before the cart was ready or is refreshing the page after they've confirmed the 
		// order.
		if (empty($order->orderitem)) flash('error', 'There are no items in your cart.');
		if (!expQueue::isQueueEmpty('error')) redirect_to(array('controller'=>'store', 'action'=>'showall'));
		
		// set the gift comments
		$order->update($this->params);
		
		// get the biling & shipping info
		$shipping = new shipping();
		$billing = new billing();

        // finalize the total to bill
        $order->calculateFinalTotal();
        
		// call the billing calculators process method - this will handle saving the billing options to the database.
		$result = $billing->calculator->process($billing->billingmethod, exponent_sessions_get('billing_options'));

        if (empty($result->errorCode)) {
            // save out the cart total to the database		
		    $billing->billingmethod->update(array('billing_cost'=>$order->finalTotal));

		    // set the invoice number and purchase date in the order table..this finializes the order
		    //$invoice_num = $db->max('orders', 'invoice_id') + 1;
		    //if ($invoice_num < ecomconfig::getConfig('starting_invoice_number')) $invoice_num += ecomconfig::getConfig('starting_invoice_number');
		    
		    // get the first order status and set it for this order
		    $order->update(array('invoice_id'=>$order->getInvoiceNumber(), 'purchased'=>time(), 'updated'=>time(), 'comment'=>serialize($comment)));
		    $order->setDefaultStatus();
		    $order->refresh();
		
            // run each items process callback function
		    foreach($order->orderitem as $item) {
		        $product = new $item->product_type($item->product_id);
		        $product->process($item);
		    }
		
            $billing->calculator->postProcess();
            
        } else {
            flash('error', 'An error was encountered while processing your transaction.<br /><br />'.$result->message);
            redirect_to(array('controller'=>'cart', 'action'=>'checkout'));
        }
        
        $billinginfo = $billing->calculator->userView(unserialize($billing->billingmethod->billing_options));
        
        // send email invoices to the admins & users if needed
        $invoice = renderAction(array('controller'=>'order', 'action'=>'email', 'id'=>$order->id));
        
		//assign_to_template(array('order'=>$order, 'billing'=>$billing, 'shipping'=>$shipping, 'result'=>$result, 'billinginfo'=>$billinginfo));
		flash('message', 'Your order has been submitted.');
        redirect_to(array('controller'=>'order', 'action'=>'myOrder', 'id'=>$order->id));
	}
	
	function quickPay() {
	    global $order, $user;
	    
	    if ($order->shipping_required) redirect_to(array('controller'=>'cart', 'action'=>'checkout'));
	    if (empty($order->orderitem)) flashAndFlow('error', 'There are no items in your cart.');
		
	    // if we made it here it means that the item was add to the cart. 
		expHistory::set('editable', $this->params);
		
		// call each products checkout() callback & calculate total
		foreach($order->orderitem as $item) {
			$product = new $item->product_type($item->product_id);
			$product->checkout();
		}
		
		// setup the billing & shipping calculators info
		if ($product->requiresBilling) {
    		$billing = new billing();
    		assign_to_template(array('billing'=>$billing));
        }
		
		if ($product->requiresShipping) {		    
		    $shipping = new shipping();		
		    $shipping->pricelist = $shipping->listPrices();
		    assign_to_template(array('shipping'=>$shipping));
		}
		
		assign_to_template(array('product'=>$product, 'user'=>$user, 'order'=>$order));
	}
	
	function processQuickPay() {
	    global $order, $template;
	    
	    // reuse the confirm action's template
	    $template = get_template_for_action($this, 'confirm', $this->loc);
	    
	    if (!empty($this->params['billing'])) {
	        $billing = new billing();
    		$billing->billingmethod->setAddress($this->params['billing']);
		}
		
		if (!empty($this->params['shipping'])) {
		    die('NEED TO IMPLEMENT THE SHIPPING PIECE!!');
		    $shipping = new shipping();
    		$shipping->shippingingmethod->setAddress($this->params['shipping']);
    		assign_to_template(array('shipping'=>$shipping));
		}
		
		$opts = $billing->calculator->userFormUpdate($this->params);
		$order->calculateFinalTotal();
		exponent_sessions_set('billing_options', $opts);
		assign_to_template(array(
		    'billing'=>$billing, 
		    'order'=>$order,
		    'total'=>$order->total, 
		    'billinginfo'=>$billing->calculator->userView($opts),
		    'nologin'=>1
		));
	}
	
    public function splitShipping() {
        global $user, $order;        

        expHistory::set('viewable', $this->params);        
        
        // get all the necessary addresses..shipping, billing, etc
		$address = new address(null, false, false);
		$addresses_dd = $address->dropdownByUser($user->id);
		
		if (count($addresses_dd) < 2) {
		    expHistory::set('viewable', $this->params);        
		    flash('error', 'You must have more than 1 address to split your shipment.  Please add another now.');
		    redirect_to(array('controller'=>'address','action'=>'edit'));
		}
		
		// setup the list of orderitems
		$orderitems = array();
		foreach($order->orderitem as $item) {
		    if ($item->product->requiresShipping == true) {
		        for($i=0; $i < $item->quantity; $i++) {
		            $orderitems[] = $item;
		        }
		    }
		}

        if (count($orderitems) < 2) {
            flashAndFlow('error', 'You must have a minimum of 2 items in your shopping cart in order to split your shipment.');
        }
        
        expHistory::set('viewable', $this->params);        
		assign_to_template(array('addresses_dd'=>$addresses_dd,'orderitems'=>$orderitems, 'order'=>$order));    
    }
    
    public function saveSplitShipping() {
        global $db;
        $addresses = array();
        $orderitems_to_delete = '';
        
        foreach ($this->params['orderitems'] as $id=>$address_ids) {            
            foreach($address_ids as $address_id) {
                if (empty($addresses[$address_id][$id])) {
                    $addresses[$address_id][$id] = 1;
                } else {
                    $addresses[$address_id][$id]+= 1;
                }
            } 
            
            if (!empty($orderitems_to_delete)) $orderitems_to_delete .= ',';
            $orderitems_to_delete .= $id;
        }
        
        foreach($addresses as $addy_id => $orderitems) {   
            $shippingmethod = new shippingmethod();
            $shippingmethod->setAddress($addy_id);     

            foreach($orderitems as $orderitem_id => $qty) {
                $orderitem = new orderitem($orderitem_id);                
                unset($orderitem->id);
                unset($orderitem->shippingmethods_id);
                $orderitem->shippingmethods_id = $shippingmethod->id;                
                $orderitem->quantity = $qty;
                $orderitem->save();
            }
        }
        
        $db->delete('orderitems', 'id IN ('.$orderitems_to_delete.')');
        redirect_to(array('controller'=>'cart', 'action'=>'selectShippingMethods'));
    }
    
    public function selectShippingMethods() {
        global $order;
        
        expHistory::set('editable', $this->params);
        $shipping = new shipping();
        $shippingmethod_id = $order->getShippingMethods();

        $shipping_items = array();
        foreach ($shippingmethod_id as $id) {
            $shipping_items[$id]->method = new shippingmethod($id);
            $shipping_items[$id]->orderitem = $order->getOrderitemsByShippingmethod($id);
            foreach ($shipping_items[$id]->orderitem as $key=>$item) {
                if ($item->product->requiresShipping == false) {
                    unset($shipping_items[$id]->orderitem[$key]);
                }
            }            
            
            if (empty($shipping_items[$id]->orderitem)) {
                unset($shipping_items[$id]);
            } else {
                foreach ($shipping->available_calculators as $calcid=>$name) {
                    $calc = new $name($calcid);
                    $shipping_items[$id]->prices[$calcid] = $calc->getRates($shipping_items[$id]);
                    //eDebug($shipping_items[$id]->prices[$id]);
                }
            }
        }
        
        assign_to_template(array('shipping_items'=>$shipping_items, 'shipping'=>$shipping));
    }
    
    public function saveShippingMethods() {
        global $order;
        
        $shipping = new shipping();
        $order->shippingmethods = array();
        
        // if they didn't fill out anything
        if (empty($this->params['methods'])) {
            expValidator::failAndReturnToForm("You did not pick  any shipping options", $this->params);
        }
        
        // if they don't check all the radio buttons
        if (count($this->params['methods']) < count($this->params['calcs'])) {
            expValidator::failAndReturnToForm("You must select a shipping options for all of your packages.", $this->params);
        }
        
        foreach ($this->params['methods'] as $id=>$method) {
            $cost = $this->params['cost'][$method];
	        $title = $this->params['title'][$method];
            $shippingmethod = new shippingmethod($id);
            $shippingmethod->update(array(
                'option'=>$method,
                'option_title'=>$title,
                'shipping_cost'=>$cost, 
                'shippingcalculator_id'=>$this->params['calcs'][$id],
            ));

            $order->shippingmethods[] = $shippingmethod->id;
        }
         
        redirect_to(array('controller'=>'cart', 'action'=>'checkout'));
    }
    
	function createaddress() {
		global $db, $user;
		if ($user->isLoggedIn()) {
		    // save the address, make it default if it is the users first one
			$address = new address();
			$count = $address->find('count', 'user_id='.$user->id);
			if ($count == 0) $this->params['is_default'] = 1;
			$this->params['user_id'] = $user->id;
			$address->update($this->params);
			
			// set the billing/shipping method
			if (isset($this->params['addresstype'])) {
			    if ($this->params['addresstype'] == 'shipping') {
			        $shipping = new shipping();
			        $shipping->shippingmethod->setAddress($address);
			    } elseif ($this->params['addresstype'] == 'billing') {
			        $billing = new billing();
			        $billing->billingmethod->setAddress($address);
			    }
			}
			
		}

		redirect_to(array('controller'=>'cart', 'action'=>'checkout'));
	}

	function getSpecials() {
		//STUB::flesh this function out eventually.
		return null;
	}

}

?>
