<?php

##################################################
#
# Copyright (c) 2004-2006 OIC Group, Inc.
# Created by Adam Kessler @ 05/28/2008
#
# This file is part of Exponent
#
# Exponent is free software; you can redistribute
# it and/or modify it under the terms of the GNU
# General Public License as published by the Free
# Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# GPL: http://www.gnu.org/licenses/gpl.txt
#
##################################################

class storeController extends expController {
    public $basemodel_name = 'product';
    public $useractions = array(
        'showall'=>'Show all products & categories',
        'showall_featured_products'=>'Show all featured products',
        'showallSubcategories'=>'Show subcategories to the current category.',
        'showallManufacturers'=>'Show products by manufacturer',
        'quicklinks'=>'Quick Links for Users',
        'showTopLevel'=>'Show Top Level Store Categories',
        'search_by_model_form'=>'Product Search - By Model',
    );
    
    function name() { return $this->displayname(); } //for backwards compat with old modules
    function displayname() { return "Ecommerce Store Front"; }
    function description() { return "Use this module to display products and categories of you Ecommerce store"; }
    function author() { return "Adam Kessler @ OIC Group, Inc"; }
    function hasSources() { return true; }
    function hasViews() { return true; }
    function hasContent() { return true; }
    function supportsWorkflow() { return false; }
    function isSearchable() { return true; }

    function showall() {
        global $db;
        
        expHistory::set('viewable', $this->params);

        // figure out if we need to show all categories and products or default to showing the first category.
        if (!isset($this->params['cat'])) {
            if (isset($this->params['title']) && is_string($this->params['title'])) {
                $default_id = $db->selectValue('storeCategories', 'id', "sef_url='".$this->params['title']."'");
            } elseif (ecomconfig::getConfig('show_first_category')) {
                $default_id = $db->selectValue('storeCategories', 'id', 'lft=1');
            } else {
                $default_id = 0;
            }
        } 
        
        $parent = isset($this->params['cat']) ? intval($this->params['cat']) : $default_id;
    
        // setup a couple blank models.
        $category = new storeCategory($parent);
        
        if (empty($category->is_events)) {
            $sql  = 'SELECT DISTINCT p.* FROM '.DB_TABLE_PREFIX.'_product p ';
            $sql .= 'JOIN '.DB_TABLE_PREFIX.'_product_storeCategories sc ON p.id = sc.product_id ';
            $sql .= 'WHERE sc.storecategories_id IN (';
            $sql .= 'SELECT id FROM exponent_storeCategories WHERE rgt BETWEEN '.$category->lft.' AND '.$category->rgt.')';         
            
            $order = ecomconfig::getConfig('orderby');
            $dir = ecomconfig::getConfig('orderby_dir');
        } else {
            $sql  = 'SELECT DISTINCT p.*, er.event_starttime, er.signup_cutoff FROM '.DB_TABLE_PREFIX.'_product p ';
            $sql .= 'JOIN '.DB_TABLE_PREFIX.'_product_storeCategories sc ON p.id = sc.product_id ';
            $sql .= 'JOIN '.DB_TABLE_PREFIX.'_eventregistration er ON p.product_type_id = er.id ';
            $sql .= 'WHERE sc.storecategories_id IN (';
            $sql .= 'SELECT id FROM exponent_storeCategories WHERE rgt BETWEEN '.$category->lft.' AND '.$category->rgt.')'; 
            if ($category->hide_closed_events) {
                $sql .= ' AND er.signup_cutoff > '.time();
            }        
                    
            $order = 'event_starttime';
            $dir = 'ASC';
        }
        
        if($category->find('count') > 0) {        
            $page = new expPaginator(array(
                'model_field'=>'product_type',
                'sql'=>$sql,
                'limit'=>empty($category->items_per_page) ? ecomconfig::getConfig('pagination_default') : $category->items_per_page,
                'order'=>$order,
                'dir'=>$dir,
                'controller'=>$this->params['controller'],
                'action'=>$this->params['action'],
                'columns'=>array('Model #'=>'model','Product Name'=>'title','Price'=>'base_price'),
                ));
        } else {
            $page = new expPaginator(array(
                'model_field'=>'product_type',
                'sql'=>'SELECT * FROM '.DB_TABLE_PREFIX.'_product WHERE 1',
                'limit'=>ecomconfig::getConfig('pagination_default'),
                'order'=>$order,
                'dir'=>$dir,
                'controller'=>$this->params['controller'],
                'action'=>$this->params['action'],
                'columns'=>array('Model #'=>'model','Product Name'=>'title','Price'=>'base_price'),
                ));
        }
        
        $ancestors = $category->pathToNode();   

        $categories = ($parent == 0) ? $category->getTopLevel() : $category->getChildren();
        
        exponent_sessions_set('last_ecomm_category', $category->id);
        
        assign_to_template(array('page'=>$page, 'ancestors'=>$ancestors, 'categories'=>$categories, 'current_category'=>$category));
    }
    
    function categoryBreadcrumb() {
        global $db;
        
        $parent = isset($this->params['cat']) ? intval($this->params['cat']) : exponent_sessions_get('last_ecomm_category');
    
        // setup a couple blank models.
        $category = new storeCategory($parent);            
        $ancestors = $category->pathToNode();   
        
        assign_to_template(array('ancestors'=>$ancestors));
    }

    function showallUncategorized() {
        expHistory::set('viewable', $this->params);
        
        $sql  = 'SELECT p.* FROM '.DB_TABLE_PREFIX.'_product p JOIN '.DB_TABLE_PREFIX.'_product_storeCategories ';
        $sql .= 'sc ON p.id = sc.product_id WHERE sc.storecategories_id = 0';
        
        $page = new expPaginator(array(
            'model_field'=>'product_type',
            'sql'=>$sql,
            'controller'=>$this->params['controller'],
            'action'=>$this->params['action'],
            'columns'=>array('Model #'=>'model','Product Name'=>'title','Price'=>'base_price'),
            ));
            
        assign_to_template(array('page'=>$page, 'moduletitle'=>'Uncategorized Products'));
    }
    
    function manage() {
        expHistory::set('managable', $this->params);
        $page = new expPaginator(array(
            'model'=>'product',
            'where'=>'1',
            'order'=>'title',
            'columns'=>array('Model #'=>'model','Type'=>'product_type', 'Product Name'=>'title','Price'=>'base_price')
            ));
        assign_to_template(array('page'=>$page));
    }
    
    function showallByManufacturer() {
        global $template;
        
        expHistory::set('viewable', $this->params);
        
        $page = new expPaginator(array(
            'model'=>'product',
            'where'=>'companies_id='.$this->params['id'],
            'default'=>'Product Name',
            'columns'=>array('Model #'=>'model','Product Name'=>'title','Price'=>'base_price')
            ));
            
        $company = new company($this->params['id']);
        
        assign_to_template(array('page'=>$page, 'company'=>$company));
    }

    function showallManufacturers() {
        global $db;
        expHistory::set('viewable', $this->params);
        $sql = 'SELECT comp.* FROM '.DB_TABLE_PREFIX.'_companies as comp JOIN '.DB_TABLE_PREFIX.'_product AS prod ON prod.companies_id = comp.id GROUP BY comp.title ORDER BY comp.title;';
        $manufacturers = $db->selectObjectsBySql($sql);
        assign_to_template(array('manufacturers'=>$manufacturers));
    }
    
    function show() {
        global $db, $order, $template;
        expHistory::set('viewable', $this->params);        
        $classname = $db->selectValue('product', 'product_type', 'id='.$this->params['id']);
        $product = new $classname($this->params['id']);
        $product_type = new $product->product_type($product->id);
        $tpl = $product_type->getForm('show');
        if (!empty($tpl)) $template = new controllerTemplate($this, $tpl);
        assign_to_template(array('product'=>$product, 'last_category'=>$order->lastcat));
    }

    function showByModel() {
        global $order, $template, $db;
        
        expHistory::set('viewable', $this->params);
        $product = new product();
        $model = $product->find("first", 'model="'.$this->params['model'].'"');
        //eDebug($model);
        $product_type = new $model->product_type($model->id);
        //eDebug($product_type);
        $tpl = $product_type->getForm('show');
        if (!empty($tpl)) $template = new controllerTemplate($this, $tpl);
        //eDebug($template);
        assign_to_template(array('product'=>$product_type, 'last_category'=>$order->lastcat));
    }

    function showByTitle() {
        global $order, $template;
        
        expHistory::set('viewable', $this->params);
        $product = new product(addslashes($this->params['title']));
        $product_type = new $product->product_type($product->id);
        $tpl = $product_type->getForm('show');
        if (!empty($tpl)) $template = new controllerTemplate($this, $tpl);

        assign_to_template(array('product'=>$product_type, 'last_category'=>$order->lastcat));
    }

    function showallSubcategories() {
        global $db;
        
        expHistory::set('viewable', $this->params);
        $parent = isset($_REQUEST['cat']) ? $_REQUEST['cat'] : exponent_sessions_get('last_ecomm_category');
        $category = new storeCategory($parent);
        $categories = $category->getEcomSubcategories();
        $ancestors = $category->pathToNode();
        assign_to_template(array('categories'=>$categories, 'ancestors'=>$ancestors, 'category'=>$category));   
    }

    function showall_featured_products() {
        $order = 'title';
        $dir = 'ASC';
        
        $page = new expPaginator(array(
                'model_field'=>'product_type',
                'sql'=>'SELECT * FROM '.DB_TABLE_PREFIX.'_product WHERE is_featured=1',
                'limit'=>ecomconfig::getConfig('pagination_default'),
                'order'=>$order,
                'dir'=>$dir,
                'controller'=>$this->params['controller'],
                'action'=>$this->params['action'],
                'columns'=>array('Model #'=>'model','Product Name'=>'title','Price'=>'base_price'),
                ));
                
        assign_to_template(array('page'=>$page));   
    }
    
    function showTopLevel() {
        $category = new storeCategory();
        $categories = $category->getEcomSubcategories();
        //$curcat = isset($this->params['cat']) ? $this->params['cat'] : "";
        $curcat = exponent_sessions_get('last_ecomm_category');
        assign_to_template(array('categories'=>$categories,'cat'=>$curcat));
    }
    
    function billing_config() {

    }
    
    function addContentToSearch() {
        global $db, $router;
        $model = new $this->basemodel_name();
        
        $total = $db->countObjects($model->table);
        
        $count = 1;
        for($i=0;$i<$total;$i+=100) {
            $orderby = 'id LIMIT '.($i+1).', 100';
            $content = $db->selectArrays($model->table, null, $orderby);            
           
            foreach ($content as $cnt) {
                $origid = $cnt['id'];
                unset($cnt['id']);
                $cnt['title'] = $cnt['title'].' - Model# '.$cnt['model'];
                $search_record = new search($cnt, false, false);
                $search_record->posted = empty($cnt['created_at']) ? null : $cnt['created_at'];
                $search_record->view_link = $router->makeLink(array('controller'=>$this->baseclassname, 'action'=>'showByTitle', 'title'=>$cnt['sef_url']));
                $search_record->ref_module = $this->baseclassname;
                $search_record->original_id = $origid;
                //$search_record->location_data = serialize($this->loc);
                $search_record->save();
                
                $count += 1;
                ob_flush();
                flush();
            }
        }
    }
    
    function search_by_model_form() {
        //do nothing...just show the view.
    }
    
    function search_by_model() {
        // get the search terms
        $terms = $this->params['search_string'];

        $sql = "model like '%".$terms."%'";

        $page = new expPaginator(array(
			'model'=>'product',
			'controller'=>$this->params['controller'],
			'action'=>$this->params['action'],
			'where'=>$sql,
			'order'=>'title',
			'dir'=>'DESC',
			'columns'=>array('Model #'=>'model','Product Name'=>'title','Price'=>'base_price'),
			));
        
        assign_to_template(array('page'=>$page, 'terms'=>$terms));
    }
    
    function edit() {
        global $db;
    
        expHistory::set('editable', $this->params);
        
        // first we need to figure out what type of ecomm product we are dealing with
        if (!empty($this->params['id'])) {
            // if we have an id lets pull the product type from the products table.
            $product_type = $db->selectValue('product', 'product_type', 'id='.$this->params['id']);
        } else {
            if (empty($this->params['product_type'])) redirect_to(array('controller'=>'store', 'action'=>picktype));
            $product_type = $this->params['product_type'];
        }
        
        $record = new $product_type($this->params['id']);
        
        // get the product options and send them to the form
        $editable_options = array();
        
        $mastergroups = $db->selectExpObjects('optiongroup_master', null, 'optiongroup_master');
        foreach($mastergroups as $mastergroup) {
            // if this optiongroup_master has already been made into an option group for this product
            // then we will grab that record now..if not, we will make a new one.
            $grouprec = $db->selectArray('optiongroup', 'optiongroup_master_id='.$mastergroup->id.' AND product_id='.$record->id);
            //eDebug($grouprec);
            if (empty($grouprec)) {
                $grouprec['optiongroup_master_id'] = $mastergroup->id;
                $grouprec['title'] = $mastergroup->title;
                $group = new optiongroup($grouprec);
            } else {
                $group = new optiongroup($grouprec['id']);
            }

            $editable_options[$group->title] = $group;

            if (empty($group->option)) {
                foreach ($mastergroup->option_master as $optionmaster) {
                    $opt = new option(array('title'=>$optionmaster->title, 'option_master_id'=>$optionmaster->id), false, false);
                    $editable_options[$group->title]->options[] = $opt;
                }
            } else {
                if (count($group->option) == count($mastergroup->option_master)) {                
                    $editable_options[$group->title]->options = $group->option;
                } else {
                    // check for any new options or deleted since the last time we edited this product
                    foreach ($mastergroup->option_master as $optionmaster) {
                        $opt_id = $db->selectValue('option', 'id', 'option_master_id='.$optionmaster->id." AND product_id=".$record->id);
                        if (empty($opt_id)) {
                            $opt = new option(array('title'=>$optionmaster->title, 'option_master_id'=>$optionmaster->id), false, false);                            
                        } else {
                            $opt = new option($opt_id);
                        }
                        
                        $editable_options[$group->title]->options[] = $opt;
                    }
                }
            }
        }
        
        // get the shipping options and their methods
        $shipping = new shipping();
        foreach ($shipping->available_calculators as $calcid=>$name) {
            $calc = new $name($calcid);
            $shipping_services[$calcid] = $calc->title;
            $shipping_methods[$calcid] = $calc->availableMethods();
        }
        
#        eDebug($shipping_services);
#        eDebug($shipping_methods);

        assign_to_template(array(
            'record'=>$record, 
            'form'=>$record->getForm('edit'), 
            'optiongroups'=>$editable_options, 
            'shipping_services'=>$shipping_services,
            'shipping_methods'=>$shipping_methods
        ));
    }
    
    function picktype() {
        $prodfiles = $this->getProductTypes();
        $products = array();
        foreach($prodfiles as $filepath=>$classname) {
            $prodObj = new $classname();
            $products[$classname] = $prodObj->product_name;
        }
        assign_to_template(array('product_types'=>$products));
    }
    
    function update() {
        global $db;
        
        $product_type = isset($this->params['product_type']) ? $this->params['product_type'] : 'product';
        $record = new $product_type();
        
        // find required shipping method if needed
        if ($this->params['required_shipping_calculator_id'] > 0) {
            $record->required_shipping_method = $this->params['required_shipping_methods'][$this->params['required_shipping_calculator_id']];
        } else {
            $this->params['required_shipping_calculator_id'] = 0;
        }
        
        $record->update($this->params);
        
        if (isset($record->id)) {
            
            $db->delete('product_storeCategories', 'product_id='.$record->id);
            
            // if there are no categories specified we'll set this to the 0 category..meaning uncategorized'
            if (empty($this->params['storeCategory'])) $this->params['storeCategory'] = array(0);
            
            // loop and save the categories.
            foreach ($this->params['storeCategory'] as $cat) {
                $assoc->storecategories_id = $cat;
                $assoc->product_id = $record->id;
                $assoc->product_type = $product_type;
                $db->insertObject($assoc, 'product_storeCategories');
            }
            
            if (!empty($this->params['optiongroups'])) {
                foreach ($this->params['optiongroups'] as $title=>$group) {
                    $optiongroup = new  optiongroup($group);
                    $optiongroup->product_id = $record->id;                                
                    $optiongroup->save();
                    
                    foreach ($this->params['optiongroups'][$title]['options'] as $opt_title=>$opt) {
                        $opt['product_id'] = $record->id;
                        $opt['is_default'] = false;
                        $opt['title'] = $opt_title;
                        $opt['optiongroup_id'] = $optiongroup->id;
                        if (isset($this->params['defaults'][$title]) && $this->params['defaults'][$title] == $opt['title']) {
                            $opt['is_default'] = true;
                        }
                        
                        $option = new option($opt);                    
                        $option->save();
                    }
                }
            }
            
        }        
        
        expHistory::back();
    }
    
    function delete() {
        global $db;
        
        if (empty($this->params['id'])) return false;
        $product_type = $db->selectValue('product', 'product_type', 'id='.$this->params['id']);
        $product = new $product_type($this->params['id']);
        if (!empty($product->product_type_id)) {
            $db->delete($product_type, 'id='.$product->product_type_id);
        }
        $db->delete('product_storeCategories', 'product_id='.$this->params['id'].' AND product_type="'.$product_type.'"');
        $product->delete();
        expHistory::back();
    }
    
    function quicklinks() {
        //we need to get the total items in the cart so that if the user at least 1 item in order to check out.
        
        $itemcount = 1;
        //eDebug($itemcount);
        assign_to_template(array("itemcount"=>$itemcount));
    }
    
    public function getProductTypes() {        
	    $paths = array(
	        BASE.'framework/modules/ecommerce/products/datatypes',
	    );
	
	    $products = array();
	    foreach ($paths as $path) {
	        if (is_readable($path)) {
                $dh = opendir($path);
                while (($file = readdir($dh)) !== false) {
                    if (is_readable($path.'/'.$file) && substr($file, -4) == '.php') {
	                    $classname = substr($file, 0, -4);
	                    $products[$path.'/'.$file] = $classname;
                    }
                }
            }
        }
        
        return $products;
    }
    
    function metainfo() {
        global $router;
        if (empty($router->params['action'])) return false;
        
        // figure out what metadata to pass back based on the action we are in.
        $action = $_REQUEST['action'];
        $metainfo = array('title'=>'', 'keywords'=>'', 'description'=>'');
        switch($action) {
            case 'show':
            case 'showByTitle':
                $prod = new product(isset($_REQUEST['title']) ? $_REQUEST['title']: $_REQUEST['id']);
                if (!empty($prod)) {
                    $metainfo['title'] = empty($prod->meta_title) ? $prod->title : $prod->meta_title;
                    $metainfo['keywords'] = empty($prod->meta_keywords) ? $prod->title : strip_tags($prod->meta_keywords);
                    $metainfo['description'] = empty($prod->meta_description) ? strip_tags($prod->body) : strip_tags($prod->meta_description);
                }              
            break;
            default:
                $metainfo = array('title'=>$this->displayname()." - ".SITE_TITLE, 'keywords'=>SITE_KEYWORDS, 'description'=>SITE_DESCRIPTION);
        }
        
        // Remove any quotes if there are any.
        $metainfo['description'] = str_replace('"', '', $metainfo['description']);
        $metainfo['keywords'] = str_replace('"', '', $metainfo['keywords']);
        
        return $metainfo;
    }
}

?>
