<?php

##################################################
#
# Copyright (c) 2007-2008 OIC Group, Inc.
# Created by Adam Kessler @ 09/06/2007
#
# This file is part of Acorn Web API
#
# Exponent is free software; you can redistribute
# it and/or modify it under the terms of the GNU
# General Public License as published by the Free
# Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# GPL: http://www.gnu.org/licenses/gpl.txt
#
##################################################

class product extends expRecord {
	public $table = 'product';
	public $has_one = array('company');
	public $has_many = array('optiongroup');
	public $has_and_belongs_to_many = array('storeCategory');

    public $product_name = 'Product';
    public $product_type = 'product';
    public $requiresShipping =true; 
	public $requiresBilling = true; 
    public $isQuantityAdjustable = true;
    
    public $quantity_display = array(
            0=>'Always available even if out of stock.',
            1=>'Available but shown as backordered if out of stock.',
            2=>'Unavailable if out of stock.',
            3=>'Show as "Call for Price".'
    );
    
    public $quantity_discount_items_modifiers = array('gte'=>'Equal to or more than', 'gt'=>'More than');
    public $quantity_discount_amount_modifiers = array('$'=>'$', '%'=>'%');
    
	public function __construct($params=array(), $get_assoc=true, $get_attached=true) {
	    global $db;
		parent::__construct($params, $get_assoc, $get_attached);
		$this->extra_fields = expUnserialize($this->extra_fields);
		$this->price = $this->getPrice();
	}
	
	function incrementQuantity($oldval) {
		return ++$oldval;
	}
	
	function updateQuantity($newval) {
		return $newval;
	}
	
	function getPrice($orderitem=null) {
		if ($this->use_special_price) {
			return $this->special_price;
		} else {
			return $this->base_price;
		}
	}
	
	function getDefaultQuantity() {
		//TMP: Make this actually do something.
		return 1;
	}

	function updateCart() {
		// nothing to do for this callback.
	}

	function checkout() {
		// nothing to do for this callback.
	}

	function addToCart($params) {
	    global $order;
	    $price = $this->getPrice();
	    $options = array();
	    foreach ($this->optiongroup as $og) {
	        if ($og->required && empty($params['options'][$og->id])) {
	            flash('message', $this->title.' requires some options to be selected before you can add it to your cart.');
	            redirect_to(array('controller'=>store, 'action'=>'show', 'id'=>$this->id));
	        }
	        if (!empty($params['options'][$og->id])) {
	            foreach ($params['options'][$og->id] as $opt_id) {
	                $selected_option = new option($opt_id);
	                $cost = $selected_option->modtype == '$' ? $selected_option->amount :  $this->getPrice() * ($selected_option->amount * .01);
	                $cost = $selected_option->updown == '+' ? $cost : $cost * -1; 
	                $price = $price + $cost;
	                $options[] = array($selected_option->id,$selected_option->title,$selected_option->modtype,$selected_option->updown,$selected_option->amount);
	            }
	        }
	    }
	    
		// add the product to the cart.
		$params['options'] = serialize($options);
		$item = new orderitem($params);
		$item->products_price = $price;
		$item->options = serialize($options);
		
		$sm = $order->getCurrentShippingMethod();
		$item->shippingmethods_id = $sm->id;
		$item->save();
		return true;
	}

    public function removeItem($item) {
        return true;
    }
    
    public function process($item) {
        $this->quantity = $this->quantity - $item->quantity;
        $this->save();
    }
    
    public function optionDropdown($key, $display_price_as) {
	    $items = array();	    
	    
	    foreach ($this->optiongroup as $index=>$group) {
	        if ($group->title == $key) {
                foreach($group->option as $option) {
                    if ($option->enable == true) {        
                        $text = $option->title;
                        
                        $price = '';
                        if (isset($option->amount)) {
                            if ($option->modtype == '%') {
                                $diff = ($this->getPrice() * ($option->amount * .01)) + $this->getPrice();
                            } else {
                                $diff = $option->amount;
                            }
                            
                            if ($display_price_as == 'total') {
                                $newprice = ($option->updown == '+') ? ($this->getPrice() + $diff) : ($this->getPrice() - $diff);
                                $price = ' ($'.number_format($newprice, 2).')';                         
                            } else {
                                $diff = '$'.number_format($diff, 2);
                                $price = ' ('.$option->updown.$diff.')';
                            }
                            
                        }                        
                        
                        $items[$option->id] = $text.$price;
                    }
                }
            }
        }
        return $items;
    }
    
    public function formatExtraData($item) {
        $viewname = $this->getForm('formatExtraData');
        if (!$viewname) return null;

        $view = new controllerTemplate($this, $viewname);
	    $view->assign('extra_data', expUnserialize($item->extra_data));
        return $view->render();
    }
    
    public function storeListing() {
        $viewname = $this->getForm('storeListing');
        if (!$viewname) return null;
        
        $view = new controllerTemplate($this, $viewname);
	    $view->assign('listing', $this);
        return $view->render();
    }
    
    public function cartSummary($item) {
        $viewname = $this->getForm('cartSummary');
        if (!$viewname) return null;
        
        $options = expUnserialize($item->options);
        $view = new controllerTemplate($this, $viewname);
	    $view->assign('product', $this);
	    $view->assign('item', $item);
	    $view->assign('options', $options);
        return $view->render('cartSummary');
    }

    
    public function getForm($form) {        
        $dirs = array(
            BASE.'themes/'.DISPLAY_THEME_REAL.'/modules/ecommerce/products/views/'.$this->product_type.'/',
            BASE.'framework/modules/ecommerce/products/views/'.$this->product_type.'/',
            BASE.'themes/'.DISPLAY_THEME_REAL.'/modules/ecommerce/products/views/product/',
            BASE.'framework/modules/ecommerce/products/views/product/',
        );
        
        foreach ($dirs as $dir) {
            if (file_exists($dir.$form.'.tpl')) return $dir.$form.'.tpl';    
        }
        
        return false;
    }
    
    public function getViewDir() {        
        return 'modules/ecommerce/products/views/';
    }
    
    public function beforeSave() {
        if (is_array($this->extra_fields)) $this->extra_fields = serialize($this->extra_fields);
    }
    
    protected function getAttachableItems() {
        if  ($this->classname != $this->product_type) $this->classname = $this->product_type;
        parent::getAttachableItems();
    }
}
?>
