<?php

##################################################
#
# Copyright (c) 2004-2008 OIC Group, Inc.
# Written and Designed by Adam Kessler
#
# This file is part of Exponent
#
# Exponent is free software; you can redistribute
# it and/or modify it under the terms of the GNU
# General Public License as published by the Free
# Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# GPL: http://www.gnu.org/licenses/gpl.txt
#
##################################################

class migrationController extends expController {
    //public $basemodel_name = '';
    protected $permissions = array('manage'=>'Manage', 'analyze'=>'Analyze Data', 'migrate'=>'Migrate Data','configure'=>'Configure');
    public $useractions = array('showall'=>'Show all');
    
    // this is a list of modules that we can convert to exp2 type modules.
    public $new_modules = array(
        'textmodule'=>'textController',
        'newsmodule'=>'newsController',
        'resourcesmodule'=>'filedownloadController',
        //'addressbookmodule'=>'address'
    );
    
    // these are modules that have either been deprecated or have no content to migrate
    public $deprecated_modules = array(
        'contactmodule',
        'imagemanagermodule',
        'imageworkshopmodule',
        'inboxmodule',
        'slideshowmodule',
        'addressbookmodule',
    );
    
    function name() { return $this->displayname(); } //for backwards compat with old modules
    function displayname() { return "Content Migration Controller"; }
    function description() { return "Use this module to pull Exponent 1 style content from your old site."; }
    function author() { return "Adam Kessler - OIC Group, Inc"; }
    function hasSources() { return false; }
    function hasViews() { return true; }
    function hasContent() { return false; }
    function supportsWorkflow() { return false; }
    function isSearchable() { return false; }
    
    public function analyze_site() {
        global $db;
        //$containers = $db->selectObjects('container', 'external="N;"');
        //eDebug($containers);
        $old_db = $this->connect();

        $sql  = 'SELECT module, COUNT(module) as count FROM '.$this->config['prefix'].'_sectionref WHERE is_original=1 GROUP BY module';
        $modules = $old_db->selectObjectsBySql($sql);
        for($i=0; $i<count($modules); $i++) {
            if (array_key_exists($modules[$i]->module, $this->new_modules)) {
                $modules[$i]->action = '<span style="color:green;">Converting content to '.$this->new_modules[$modules[$i]->module]."</span>";
            } elseif (in_array($modules[$i]->module, $this->deprecated_modules)) {
                $modules[$i]->action = '<span style="color:red;font-weight:bold">This module is deprecated.</span>';
            } else {
                $modules[$i]->action = 'Migrating as is.';
            }
        }
        
        assign_to_template(array('modules'=>$modules));
    }
    
    public function migrate_content() {
        global $db;
        
        $old_db = $this->connect();
        $db->delete('sectionref');
        $db->delete('locationref');
        $db->delete('container');
        $db->delete('text');
        $db->delete('news');
        $db->delete('filedownloads');
        $db->delete('expFiles');
        $db->delete('content_expFiles');
        $db->delete('calendar');
        $db->delete('calendarmodule_config');
        $db->delete('eventdate');
        
        echo "Cleaned Database tables before starting<br>";
        
        // pull the sectionref data
        $secref = $old_db->selectObjects('sectionref');
        foreach ($secref as $sr) {
            if (array_key_exists($sr->module, $this->new_modules)) {
                $sr->module = $this->new_modules[$sr->module];
                $db->insertObject($sr, 'sectionref');
            } elseif (in_array($sr->module, $this->deprecated_modules)) {
                // do nothing...we don't want this module
            } else {
                $db->insertObject($sr, 'sectionref');
            }            
        }
        echo "Imported Sectionref data<br>";
        
        // pull the locationred data
        $locref = $old_db->selectObjects('locationref');
        foreach ($locref as $lr) {
            if (array_key_exists($lr->module, $this->new_modules)) {
                $lr->module = $this->new_modules[$lr->module];
                $db->insertObject($lr, 'locationref');
            } elseif (in_array($lr->module, $this->deprecated_modules)) {
                // do nothing...we don't want this module
            } else {
                $db->insertObject($lr, 'locationref');
            }             
        }
        echo "Imported locationref data<br>";
        
        // pull over all the container modules
        $containers = $old_db->selectObjects('container', 'external="N;"');
        foreach ($containers as $cont) {
            $db->insertObject($cont, 'container');
        }
        echo "Imported containermodules<br>";
        
        // this will pull all the old modules.  if we have a exp2 equivalent module
        // we will convert it to the new type of module before pulling.
        $modules = $old_db->selectObjects('container', 'external != "N;"');
        foreach($modules as $module) {
            $iloc = expUnserialize($module->internal);
            if (array_key_exists($iloc->mod, $this->new_modules)) {
                $this->convert($iloc, $module);                
            } elseif (in_array($iloc->mod, $this->deprecated_modules)) {
                // do nothing...we don't want this module
            } else {
                $db->insertObject($module, 'container');
                $this->pulldata($iloc, $module);
            } 
        }
        
        // import the files
        $oldfiles = $old_db->selectArrays('file');
        foreach ($oldfiles as $oldfile) {
            unset($oldfile['id']);
            $file = new expFile($oldfile);
            $file->save();
        }
       
        expSession::clearUserCache();
    }
    
    private function pulldata($iloc, $module) {
        global $db;
        $old_db = $this->connect();
        
        switch ($iloc->mod) {
            case 'calendarmodule':
                $events = $old_db->selectObjects('eventdate', "location_data='".serialize($iloc)."'");
                foreach($events as $event) {
                    $db->insertObject($event, 'eventdate');
                }
                $cals = $old_db->selectObjects('calendar', "location_data='".serialize($iloc)."'");
                foreach($cals as $cal) {
                    unset($cal->allow_registration);
                    unset($cal->registration_limit);
                    unset($cal->registration_allow_multiple);
                    unset($cal->registration_cutoff);
                    unset($cal->registration_price);
                    unset($cal->registration_count);
                    $db->insertObject($cal, 'calendar');
                }
                $configs = $old_db->selectObjects('calendarmodule_config', "location_data='".serialize($iloc)."'");
                foreach ($configs as $config) {
                    $db->insertObject($config, 'calendarmodule_config');
                }
            break;
            case 'bbmodule':
                $tables = array('bb_board', 'bb_boardmonitor','bb_post','bb_rank', 'bb_ranks_users', 'bb_threadmonitor', 'bb_user', 'bb_user_board_visit', 'bbmodule_config');
                
                foreach($tables as $table) {
                    $datas = $old_db->selectObjects($table);
                    foreach($datas as $data) {
                        $db->insertObject($data, $table);
                    }
                }
                
            break;
        }
    }
    
    private function convert($iloc, $module) {
        global $db;
        $old_db = $this->connect();
        
        switch ($iloc->mod) {
            case 'textmodule':
                echo "Converting ".$iloc->mod."<br>";
                $iloc->mod = $this->new_modules[$iloc->mod];
                $module->internal = serialize($iloc);
                $module->action = 'showall';
                $module->view = 'showall';
                $db->insertObject($module, 'container');
                
                $iloc->mod = 'textmodule';
                $textitems = $old_db->selectObjects('textitem', "location_data='".serialize($iloc)."'");
                foreach ($textitems as $ti) {
                    $text = new text();
                    $loc = expUnserialize($ti->location_data);
                    $loc->mod = "text";
                    $text->location_data = serialize($loc);
                    $text->body = $ti->text;
                    $text->save();
                }
            break;
            case 'newsmodule':
                echo "Converting ".$iloc->mod."<br>";
                $iloc->mod = $this->new_modules[$iloc->mod];
                $module->internal = serialize($iloc);
                $module->action = 'showall';
                $module->view = 'showall';
                $db->insertObject($module, 'container');
                
                $iloc->mod = 'newsmodule';
                $newsitems = $old_db->selectArrays('newsitem', "location_data='".serialize($iloc)."'");
                foreach ($newsitems as $ni) {
                    unset($ni['id']);
                    $news = new news($ni);                   
                    $loc = expUnserialize($ni['location_data']);
                    $loc->mod = "news";
                    $news->location_data = serialize($loc);
                    $news->created_at = $ni['posted'];
                    $news->edited_at = $ni['edited'];                    
                    $news->save();
                    if (!empty($ni['file_id'])) {
                        $oldfile = $old_db->selectArray('file', 'id='.$ni['file_id']);
                        unset($oldfile['id']);
                        $file = new expFile($oldfile);
                        $file->save();
                        $news->attachitem($file);
                    }
                }
            break;
            case 'resourcesmodule':
                echo "Converting ".$iloc->mod."<br>";
                $iloc->mod = $this->new_modules[$iloc->mod];
                $module->internal = serialize($iloc);
                $module->action = 'showall';
                $module->view = 'showall';
                $db->insertObject($module, 'container');
                
                $iloc->mod = 'resourcesmodule';
                $resourceitems = $old_db->selectArrays('resourceitem', "location_data='".serialize($iloc)."'");
                foreach ($resourceitems as $ri) {
                    unset($ri['id']);
                    $filedownload = new filedownload($ri);                   
                    $loc = expUnserialize($ri['location_data']);
                    $loc->mod = "filedownload";
                    $filedownload->title = $ri['name'];
                    $filedownload->body = $ri['description'];
                    $filedownload->downloads = $ri['num_downloads'];
                    $filedownload->location_data = serialize($loc);
                    $filedownload->created_at = $ri['posted'];
                    $filedownload->edited_at = $ri['edited'];                    
                    $filedownload->save();
                    if (!empty($ri['file_id'])) {
                        $oldfile = $old_db->selectArray('file', 'id='.$ri['file_id']);
                        unset($oldfile['id']);
                        $file = new expFile($oldfile);
                        $file->save();
                        $filedownload->attachitem($file);
                    }
                }
            break;
            default:
                echo '<span style="color:red; font-weight:bold;">No converter for '.$iloc->mod.'</span><br>';
            break;
        }
    }
    
    
    
    public function manage_pages() {
        expHistory::set('managable', $this->params);
        $old_db = $this->connect();
        $pages = $old_db->selectObjects('section');
        assign_to_template(array('pages'=>$pages));
    }
    
    public function migrate_pages() {
        global $db;
        
        $successful = 0;
        $failed     = 0;
        $old_db = $this->connect();
        foreach($this->params['pages'] as $pageid) {
            $page = $old_db->selectObject('section', 'id='.$pageid);
            $ret = $db->insertObject($page, 'section');
            if (empty($ret)) {
                $failed += 1;
            } else {
                $successful += 1;
            }
        }
        
        flash ('message', $successful.' pages were imported from '.$this->config['database']);
        if ($failed > 0) {
            flash('error', $failed.' pages could not be imported from '.$this->config['database'].' This is usually because a page with the same ID already exists in the database you importing to.');
        }
        
        expSession::clearUserCache();
        expHistory::back();
    }
    
    private function connect() {
        // check for required info...then make the DB connection.
        if (
            empty($this->config['username']) || 
            empty($this->config['password']) ||
            empty($this->config['database']) ||
            empty($this->config['server']) ||
            empty($this->config['prefix']) ||
            empty($this->config['port'])
        ) {
            flash ('error', 'You are missing some required database connectin information.  Please enter DB information.');
            redirect_to (array('controller'=>'migration', 'action'=>'configure'));
        }
        
       $database = exponent_database_connect($this->config['username'],$this->config['password'],$this->config['server'].':'.$this->config['port'],$this->config['database']);
       
       if (empty($database->havedb)) {
            flash ('error', 'An error was encountered trying to connect to the database you specified. Please check your DB config.');
            redirect_to (array('controller'=>'migration', 'action'=>'configure'));
       } 
       
       $database->prefix = $this->config['prefix']. '_';;
       return $database;
    }
}

?>
